//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/IntPol.h,v $
// $Id: IntPol.h,v 1.6 2004/04/20 22:04:48 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_INTPOL_H	// prevent multiple includes
#define ETECH_INTPOL_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Types.h>
#include <vector>		// STL

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)


struct IntPolData
{
    Dbl x;
    Dbl y;
};

/// represents a pure virtual interpolation class
/**
 {\tt IntPol} represents an pure interpolation class.
 */
class IntPol
{
  public:
    /// constructor
    IntPol(IntPolData* data, unsigned int size);

    /// virtual destructor
    virtual ~IntPol();

    /// return interpolated value
    virtual Dbl getY(Dbl x) const;

  protected:
    std::vector<IntPolData> table;

    virtual Dbl interpolate(Dbl x, const IntPolData& p, 
			    const IntPolData& q) const = 0;
}; // class IntPol


/// represents a linear interpolation class
/**
 {\tt IntPolLin1Lin2} represents a linear interpolation class.
 */
class IntPolLin1Lin2: public IntPol
{
  public:
    /// constructor
    IntPolLin1Lin2(IntPolData* data, unsigned int size)
	: IntPol(data, size) {}
  protected:
    virtual Dbl interpolate(Dbl x, const IntPolData& p, 
			    const IntPolData& q) const;
}; // class IntPolLin1Lin2


/// represents a logarithmic interpolation class
/**
 {\tt IntPolLog1Lin2} represents a single logarithmic interpolation class.
 */
class IntPolLog1Lin2: public IntPol
{
  public:
    /// constructor
    IntPolLog1Lin2(IntPolData* data, unsigned int size)
	: IntPol(data, size) {}
  protected:
    virtual Dbl interpolate(Dbl x, const IntPolData& p, 
			    const IntPolData& q) const;
}; // class IntPolLog1Lin2


END_NAMESPACE

#endif // ETECH_INTPOL_H
