//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/NoneInvAmp.h,v $
// $Id: NoneInvAmp.h,v 1.6 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_NoneInvAmp_H	// prevent multiple includes
#define ETECH_NoneInvAmp_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Amplifier.h>
#include <AH/Etech/OpAmp.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents a none inverting amplifier
/**
 {\tt NoneInvAmp} is a none inverting amplifier which uses an
 operational amplifer. It can be used both in DC (direct current) and in 
 AC (alternate current) applications.
 */
class NoneInvAmp: public Amplifier
{
    OpAmp     op;	// used operational amplifier
    Impedance Z1;
    Impedance Z2;
    Impedance* Zin;

  protected:

    /// get open loop amplification at frequency {\tt freq}
    virtual const Cplx* getAd(Dbl freq);
    
  public:
    /// constructor:
    NoneInvAmp(const Impedance& z1, const Impedance& z2) : 
	op(), Z1(z1), Z2(z2), Zin(0) {}
    /// constructor:
    NoneInvAmp(const OpAmp& opamp, const Impedance& z1, const Impedance& z2) : 
	op(opamp), Z1(z1), Z2(z2), Zin(0) {}
    /// constructor:
    NoneInvAmp(const OpAmp& opamp, const Impedance& z1, const Impedance& z2,
	       const Impedance& zi) : 
	op(opamp), Z1(z1), Z2(z2) { Zin = new Impedance(zi); }

    /// destructor:
    virtual ~NoneInvAmp() { if (Zin) delete Zin; }

    /// get amplification at frequency {\tt freq}
    virtual Cplx getA(Dbl freq);
    /// get amplification at frequency {\tt freq} with {\tt load}
    virtual Cplx getA(Dbl freq, const Impedance& load);

//    /// overloaded operators:
//    friend ostream& operator << (ostream &out, const NoneInvAmp &z);

}; // class NoneInvAmp


END_NAMESPACE

#endif // ETECH_NoneInvAmp_H
