//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Resistor.h,v $
// $Id: Resistor.h,v 1.9 2004/04/20 22:04:48 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_RESITOR_H	// prevent multiple includes
#define ETECH_RESITOR_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Impedance.h>

USING_NAMESPACE(Ah)

// --- forward declarations ---

class ostream;

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an ideal electrical resistor
/**
 {\tt Resistor} represents an ideal electrical resistor which can be used both 
 in DC (direct current) and in AC (alternate current) applications.
 */
class Resistor: public Impedance
{
  private:
    Dbl Resistance;

  public:
    /// constructor
    Resistor(Dbl r = 0.0) : Impedance(r), Resistance(r) {}
    /// copy constructor
    Resistor(const Resistor& r) 
	: Impedance(r.Value), Resistance(r.Resistance) {}

    /// return resistance
    Dbl getRes() const { return Resistance; }

    /// overloaded operator + for serial circuit of two resistors
    friend Resistor operator + (const Resistor &r1, const Resistor &r2)
	{ return Resistor(r1.Resistance + r2.Resistance); }

    /// overloaded operator || for parallel circuit of two resistors
    friend Resistor operator || (const Resistor &r1, const Resistor &r2)
	{ return Resistor(r1.Resistance * r2.Resistance / 
		 	  (r1.Resistance + r2.Resistance)); }

    /// overloaded operator * for multiplication of {\tt Resistor} * {\tt Dbl}
    friend Resistor operator * (const Resistor &r, Dbl k)
	{ return Resistor(r.Resistance * k); }
    /// overloaded operator * for multiplication of{\tt Dbl} * {\tt Resistor}
    friend Resistor operator * (Dbl k, const Resistor &r)
	{ return Resistor(k * r.Resistance); }

    /// overloaded operator / for division of {\tt Resistor} / {\tt Dbl}
    friend Resistor operator / (const Resistor &r, Dbl k)
	{ return Resistor(r.Resistance / k); }

    /// overloaded operator for output into streams
    friend std::ostream& operator << (std::ostream &out, const Resistor &r);

}; // class Resistor


END_NAMESPACE

#endif // ETECH_RESITOR_H

