//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/VierPol.h,v $
// $Id: VierPol.h,v 1.4 2004/04/20 22:04:48 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_VIERPOL_H	// prevent multiple includes
#define ETECH_VIERPOL_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Types.h>

USING_NAMESPACE(Ah)

// --- forward declarations ---

class ostream;

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

// --- types and classes ---

class VierPol
{
  protected:
    Cplx c11;
    Cplx c12;
    Cplx c21;
    Cplx c22;

    /// default constructor
    VierPol ()
	: c11(), c12(), c21(), c22() {}
    /// constructor
    VierPol (Cplx C11, Cplx C12, Cplx C21, Cplx C22)
	: c11(C11), c12(C12), c21(C21), c22(C22) {}
    /// destructor
    ~VierPol() {}

    /// return true if equal
    bool equal(const VierPol& vp) const
	{ return (c11 == vp.c11) && (c12 == vp.c12) && 
		 (c21 == vp.c21) && (c22 == vp.c22); }

  public:
    /// calculate and return determinante
    Cplx det() const
	{ return ((c11 * c22) - (c12 * c21)); }
    /// return true if {\tt VierPol} is valid
    bool isValid() const 
	{ return (c11.isValid() && c12.isValid() && c21.isValid() && 
		  c22.isValid()); }

    /// overloaded operator for output into streams
    friend std::ostream& operator << (std::ostream &out, const VierPol &vp);
};

// forward declarations (there are cyclic dependencies between these classes)

class VierPolZ;
class VierPolY;
class VierPolH;
class VierPolA;

// classes derived from VierPol:

class VierPolZ: public VierPol
{
  public:
    /// constructor
    VierPolZ (Cplx z11, Cplx z12, Cplx z21, Cplx z22)
	: VierPol(z11, z12, z21, z22) {}
    /// copy constructor
    VierPolZ (const VierPolZ& vp) 
	: VierPol(vp.c11, vp.c12, vp.c21, vp.c22) {}
    /// convert constructor
    VierPolZ (const VierPolY& vpy); 
    /// convert constructor
    VierPolZ (const VierPolH& vph); 
    /// convert constructor
    VierPolZ (const VierPolA& vpa); 
    /// destructor
    ~VierPolZ() {}

    /// accessors:
    Cplx z11() const { return c11; }
    Cplx z12() const { return c12; }
    Cplx z21() const { return c21; }
    Cplx z22() const { return c22; }

    /// convert VierPolY into VierPolZ
    const VierPolZ& operator = (const VierPolY& vpy);
    /// convert VierPolH into VierPolZ
    const VierPolZ& operator = (const VierPolH& vph);
    /// convert VierPolA into VierPolZ
    const VierPolZ& operator = (const VierPolA& vpa);

    /// overloaded operator == for comparison of two {\tt VierPolZ}
    friend bool operator == (const VierPolZ &vp1, const VierPolZ& vp2)
	{ return vp1.equal(vp2); }
    /// overloaded operator != for comparison of two {\tt VierPolZ}
    friend bool operator != (const VierPolZ &vp1, const VierPolZ& vp2)
	{ return !vp1.equal(vp2); }
};


class VierPolY: public VierPol
{
  public:
    /// constructor
    VierPolY (Cplx y11, Cplx y12, Cplx y21, Cplx y22)
	: VierPol(y11, y12, y21, y22) {}
    /// copy constructor
    VierPolY (const VierPolY& vp) 
	: VierPol(vp.c11, vp.c12, vp.c21, vp.c22) {}
    /// convert constructor
    VierPolY (const VierPolZ& vpz); 
    /// convert constructor
    VierPolY (const VierPolH& vph); 
    /// convert constructor
    VierPolY (const VierPolA& vpa); 
    /// destructor
    ~VierPolY() {}

    /// accessors:
    Cplx y11() const { return c11; }
    Cplx y12() const { return c12; }
    Cplx y21() const { return c21; }
    Cplx y22() const { return c22; }

    /// convert VierPolZ into VierPolY
    const VierPolY& operator = (const VierPolZ& vpz);
    /// convert VierPolH into VierPolY
    const VierPolY& operator = (const VierPolH& vph);
    /// convert VierPolA into VierPolY
    const VierPolY& operator = (const VierPolA& vpa);

    /// overloaded operator == for comparison of two {\tt VierPolY}
    friend bool operator == (const VierPolY &vp1, const VierPolY& vp2)
	{ return vp1.equal(vp2); }
    /// overloaded operator != for comparison of two {\tt VierPolY}
    friend bool operator != (const VierPolY &vp1, const VierPolY& vp2)
	{ return !vp1.equal(vp2); }
};


class VierPolH: public VierPol
{
  public:
    /// constructor
    VierPolH (Cplx h11, Cplx h12, Cplx h21, Cplx h22)
	: VierPol(h11, h12, h21, h22) {}
    /// copy constructor
    VierPolH (const VierPolH& vp) 
	: VierPol(vp.c11, vp.c12, vp.c21, vp.c22) {}
    /// convert constructor
    VierPolH (const VierPolY& vpy); 
    /// convert constructor
    VierPolH (const VierPolZ& vpz); 
    /// convert constructor
    VierPolH (const VierPolA& vpa); 
    /// destructor
    ~VierPolH() {}

    /// accessors:
    Cplx h11() const { return c11; }
    Cplx h12() const { return c12; }
    Cplx h21() const { return c21; }
    Cplx h22() const { return c22; }

    /// convert VierPolZ into VierPolH
    const VierPolH& operator = (const VierPolZ& vpz);
    /// convert VierPolY into VierPolH
    const VierPolH& operator = (const VierPolY& vpy);
    /// convert VierPolA into VierPolH
    const VierPolH& operator = (const VierPolA& vpa);

    /// overloaded operator == for comparison of two {\tt VierPolH}
    friend bool operator == (const VierPolH &vp1, const VierPolH& vp2)
	{ return vp1.equal(vp2); }
    /// overloaded operator != for comparison of two {\tt VierPolH}
    friend bool operator != (const VierPolH &vp1, const VierPolH& vp2)
	{ return !vp1.equal(vp2); }
};


class VierPolA: public VierPol
{
  public:
    /// constructor
    VierPolA (Cplx a11, Cplx a12, Cplx a21, Cplx a22)
	: VierPol(a11, a12, a21, a22) {}
    /// copy constructor
    VierPolA (const VierPolA& vp) 
	: VierPol(vp.c11, vp.c12, vp.c21, vp.c22) {}
    /// convert constructor
    VierPolA (const VierPolY& vpy); 
    /// convert constructor
    VierPolA (const VierPolH& vph); 
    /// convert constructor
    VierPolA (const VierPolZ& vpz); 
    /// destructor
    ~VierPolA() {}

    /// accessors:
    Cplx a11() const { return c11; }
    Cplx a12() const { return c12; }
    Cplx a21() const { return c21; }
    Cplx a22() const { return c22; }

    /// convert VierPolZ into VierPolA
    const VierPolA& operator = (const VierPolZ& vpz);
    /// convert VierPolH into VierPolA
    const VierPolA& operator = (const VierPolH& vph);
    /// convert VierPolY into VierPolA
    const VierPolA& operator = (const VierPolY& vpy);

    /// overloaded operator == for comparison of two {\tt VierPolA}
    friend bool operator == (const VierPolA &vp1, const VierPolA& vp2)
	{ return vp1.equal(vp2); }
    /// overloaded operator != for comparison of two {\tt VierPolA}
    friend bool operator != (const VierPolA &vp1, const VierPolA& vp2)
	{ return !vp1.equal(vp2); }
};


END_NAMESPACE

#endif // ETECH_VIERPOL_H
