//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Voltage.h,v $
// $Id: Voltage.h,v 1.6 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_VOLTAGE_H	// prevent multiple includes
#define ETECH_VOLTAGE_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/PhysUnit.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an electrical voltage
/**
 {\tt Voltage} represents an electrical voltage which can be used both in
 DC (direct current) and in AC (alternate current) applications.
 */
class Voltage: public PhysUnitCplx
{
  public:
    /// constructor
    Voltage(Dbl r = 0.0, Dbl i = 0.0) : PhysUnitCplx("V",r,i) {}
    /// constructor
    Voltage(const Cplx &val) : PhysUnitCplx("V", val) {}
    /// copy constructor
    Voltage(const Voltage& u) : PhysUnitCplx("V",u.Value) {}

    /// overloaded operator + for addition of two voltages
    friend Voltage operator + (const Voltage &u1, const Voltage &u2)
	{ return Voltage(u1.Value + u2.Value); }
    /// overloaded operator - for subtraction of two voltages
    friend Voltage operator - (const Voltage &u1, const Voltage &u2)
	{ return Voltage(u1.Value - u2.Value); }

    /// overloaded operator * for multiplication of {\tt Voltage} * {\tt Cplx}
    friend Voltage operator * (const Voltage &u1, const Cplx &k)
	{ return Voltage(u1.Value * k); }
    /// overloaded operator * for multiplication of {\tt Cplx} * {\tt Voltage}
    friend Voltage operator * (const Cplx &k, const Voltage &u1)
	{ return Voltage(k * u1.Value); }

    /// overloaded operator / for division of {\tt Voltage} / {\tt Cplx}
    friend Voltage operator / (const Voltage &u1, const Cplx &k)
	{ return Voltage(u1.Value / k); }

    /// overloaded operator == for comparison of two Voltages
    friend bool operator == (const Voltage &u1, const Voltage& u2)
	{ return (u1.Value == u2.Value); }
    /// overloaded operator != for comparison of two Voltages
    friend bool operator != (const Voltage &u1, const Voltage& u2)
	{ return (u1.Value != u2.Value); }
    /// overloaded operator < for comparison of two Voltages
    friend bool operator < (const Voltage &u1, const Voltage& u2)
	{ return (u1.Value < u2.Value); }
    /// overloaded operator > for comparison of two Voltages
    friend bool operator > (const Voltage &u1, const Voltage& u2)
	{ return (u1.Value > u2.Value); }

};


END_NAMESPACE

#endif // ETECH_VOLTAGE_H
