//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Plot.h,v $
// $Id: Plot.h,v 1.4 2004/04/20 22:25:30 ah Exp $
//
// Description: interface of class IOPlot
//
//---------------------------------------------------------------------------

#ifndef AH_IO_PLOT_H		// prevent multiple includes
#define AH_IO_PLOT_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <string>
#include <vector>

// --- types and classes ---

BEGIN_NAMESPACE(AH)


/// use <em>gnuplot</em> to plot data from within this process
/**
   <tt>AH::IOPlot</tt> uses <em>gnuplot</em> to plot data.

   Please include <em>AH/IO/Plot.h</em>.
 */
class IOPlot
{
 public:
    /// constructor: start gnuplot
    IOPlot();
    IOPlot(const std::string& plotprog);
    /// destructor: exit gnuplot
    ~IOPlot();

    /// send command to gnuplot
    void exec(const std::string& cmd);
    /// send multiple commands to gnuplot
    void exec(const std::vector<std::string>& cmds);

    /// send PLOT command to gnuplot to plot data from a file
    void plot(const std::string& file, const std::string& options);
    /// send PLOT command to gnuplot
    void plot(const std::string& options);
    /// send PLOT command to gnuplot to plot data directly
    void plot(const std::vector<std::string>& data, const std::string& options);

    /// operator () returns the pipe to gnuplot
    FILE* operator() () const { return fp; }

    /// set X range (horizontal axis)
    void setXrange(double from, double to);

    /// set Y range (vertical axis)
    void setYrange(double from, double to);

 private:
    FILE* fp;

    /// flush command pipe
    void flush();

};	// IOPlot



END_NAMESPACE

#endif // !AH_IO_PLOT_H

