//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Riff/WaveDataBuf.h,v $
// $Id: WaveDataBuf.h,v 1.3 2004/04/20 22:25:30 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_RIFF_WAVE_DATA_BUF_H		// prevent multiple includes
#define AH_IO_RIFF_WAVE_DATA_BUF_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/IO/Riff/Chunk.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <string>
#include <iostream>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)
/// buffer for RIFF wave data
/**
   <tt>IORiffWaveDataBuffer</tt> represents a buffer for RIFF wave data.

   Please include <em>AH/IO/Riff/WaveDataBuf.h</em>.
 */
class IORiffWaveDataBuffer
{
 public:
    /// constructor for buffer of <tt>n</tt> elements of size <tt>bits</tt> bits
    /**
       The constructor creates a new buffer which contains <tt>n</tt> elements
       where each element occupies <tt>bits</tt> bits.

       So far only 8, 16, 24 and 32 bits are supported. For other values
       an exception is thrown.
     */
    IORiffWaveDataBuffer(size_t n, size_t bits);

    /// destructor
    ~IORiffWaveDataBuffer() { if (buf) delete[] buf; }

    /// resize buffer for <tt>n</tt> elements
    /**
      <tt>resize(s)</tt> resizes the buffer to contains <tt>s</tt> elements:
      - if <tt>s</tt> is bigger than the current size then the old buffer
        is destroyed and a new one with the appropriate size is created.
      - if <tt>s</tt> is smaller than the current size then nothing is done
        at all.
      - if <tt>s</tt> is equal to the current size then nothing is done
        at all.
      .
      This means that the buffer does only grow but never shrinks. This
      speeds up the execution but blocks memory.
     */
    void resize(size_t n);

    /// recreate buffer for <tt>n</tt> elements
    /**
      <tt>recreate(s)</tt> resizes the buffer to contains <tt>s</tt> elements:
      - if <tt>s</tt> is not equal to the current size then the old buffer
        is destroyed and a new one with the appropriate size is created.
      - if <tt>s</tt> is equal to the current size then nothing is done
        at all.
      .
      This means that the buffer grows and shrinks according to the new 
      size <tt>s</tt>. This costs execution time but does not block memory.
     */
    void recreate(size_t n);

    /// empty buffer
    void clear()
    {
	memset(buf, 0, getSize());
    }

    /// return number of elements which can be stored in buffer
    size_t getNum() const { return curnum; }
    /// return size of buffer in bytes
    size_t getSize() const { return cursiz; }

    /// return specified entry in buffer
    int32 getEntry(size_t idx) const;

    /// return pointer to buffer
    void* operator()() const { return buf; }

 private:
    size_t	bytes;	// number of bytes per element
    size_t	curnum;	// current size of buffer in number of entries
    size_t	maxnum;	// maximal size of buffer in number of entries
    size_t	cursiz;	// current size of buffer in bytes
    size_t	maxsiz;	// maximal size of buffer in bytes
    uint8	shift;	// shift for getEntry()
    uint32	mask;	// mask for getEntry()
    uint8*	buf;	// the buffer itsself

    /// check if buffer could be created
    void checkBuf();
};

END_NAMESPACE

#endif // !AH_IO_RIFF_WAVE_DATA_BUF_H










