//---------------------------------------------------------------------------
// Copyright 2004 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHmath/DataStat.c,v $
// $Id: DataStat.c,v 1.2 2004/04/06 23:14:32 ah Exp $ 
//
// Description: Implementation of class DataStat
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: DataStat.c,v 1.2 2004/04/06 23:14:32 ah Exp $ (c) 2004 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/Math/DataStat.h>
USING_NAMESPACE(AH)

#include <AH/Error/Intern.h>
USING_NAMESPACE(AH)

BEGIN_NAMESPACE(AH)

// --- local definitions ---

#define PROFILING 0
#define TRACE 0

// --- local constants ---

// --- local variables ---

// --- local classes ---

// --- local functions ---

// --- member functions ---


// calculate mean value of data array
void DataStat::calcMean
(
    const double* data, unsigned int dataNum,
    double* outMean
)
{
    // do some checks:
    if (!outMean)
	return; // nothing to do at all
    if (0 == dataNum) 
	throw ErrorInternBadArg("dataNum = 0 in DataStat::calcMeanMaxMin()");
    if (!data)
	throw ErrorInternNullPtrArg("data in DataStat::calcMeanMaxMin()");

    // determine mean value only:
    double mean = data[0];
    for (unsigned int i = 1; i < dataNum; ++i)
    {
	mean += data[i];
    }

    *outMean = mean / (double)dataNum; // mean value

} // end of DataStat::calcMean()



// calculate maximum and minimum value of data array
void DataStat::calcMaxMin
(
    const double* data, unsigned int dataNum,
    double* outMax, double* outMin
)
{
    // do some checks:
    if (!outMax && !outMin)
	return; // nothing to do at all
    if (0 == dataNum) 
	throw ErrorInternBadArg("dataNum = 0 in DataStat::calcMeanMaxMin()");
    if (!data)
	throw ErrorInternNullPtrArg("data in DataStat::calcMeanMaxMin()");

    // determine min and max value:
    double max = data[0];
    double min = data[0];
    for (unsigned int i = 1; i < dataNum; ++i)
    {
	if (data[i] > max) max = data[i];
	if (data[i] < min) min = data[i];
    }

    if (outMax) *outMax  = max;
    if (outMin) *outMin  = min;

} // end of DataStat::calcMeanMaxMin()



// calculate mean, max and minimum value of data array
void DataStat::calcMeanMaxMin
(
    const double* data, unsigned int dataNum,
    double* outMean, double* outMax, double* outMin
)
{
    // do some checks:
    if (!outMean && !outMax && !outMin)
	return; // nothing to do at all
    if (0 == dataNum) 
	throw ErrorInternBadArg("dataNum = 0 in DataStat::calcMeanMaxMin()");
    if (!data)
	throw ErrorInternNullPtrArg("data in DataStat::calcMeanMaxMin()");

    if (!outMax && !outMin)
	calcMean(data, dataNum, outMean); // this is faster
    else if (!outMean)
	calcMaxMin(data, dataNum, outMax, outMin);  // this is faster
    else
    {
	// determine min, mean and max value:
	double mean = data[0];
	double max = data[0];
	double min = data[0];
	for (unsigned int i = 1; i < dataNum; ++i)
	{
	    mean += data[i];
	    if (data[i] > max) max = data[i];
	    if (data[i] < min) min = data[i];
	}
	
	*outMean = mean / (double)dataNum; // mean value
	if (outMax) *outMax  = max;
	if (outMin) *outMin  = min;
    }

} // end of DataStat::calcMeanMaxMin()


END_NAMESPACE
