//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: 
// $Id: 
//
// Description: implementation of class RegulaFalsi
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: RegulaFalsi.c,v 1.4 2004/04/20 22:34:39 ah Exp $ (c) 1998 Andreas Huennebeck";

#define TRACE 1

// --- necessary includes ---

#include <AH/Math/RegulaFalsi.h>
#include <math.h>
#if TRACE
#   include <iostream>
#endif

USING_NAMESPACE(Ah)

// --- local stuff ---

// --- class member implementation ---

BEGIN_NAMESPACE(Ah)


double RegulaFalsi::search
(
    double xLo,
    double xHi,
    double eps
)
{
    double yLo = f(xLo);
    double yHi = f(xHi);

    //--------------------------------
    // check limits for convergence :
    //--------------------------------
    if ((yLo * yHi) > 0)
    {
	return (xLo + xHi)/2.0;	// no convergence    
    }

    while (1)
    {
	if (fabs(yLo) < eps) return xLo;
	if (fabs(yHi) < eps) return xHi;

	double xCur = xLo - (xHi - xLo) / (yHi - yLo) * yLo;
	double yCur = f(xCur);
	if (fabs(yCur) < eps) return xCur;
	
	if (0 > (yCur * yLo))	
	{
	    xHi = xCur;
	    yHi = yCur;
	}
	else
	{
	    xLo = xCur;
	    yLo = yCur;
	}
#if TRACE
	std::cerr << "."; std::cerr.flush();
#endif
    }
}


// --- friend functions ---

// --- overloaded operators ---

END_NAMESPACE
